'''conversion.py

This module is for converting between Pythonnet representations and mastapy
representations. Should only be used internally.
'''


from typing import List, Union, Sequence, Any
from datetime import datetime
from io import BytesIO
from collections import OrderedDict
from numpy import ndarray
from packaging.version import Version

from PIL import Image

from mastapy._internal.python_net import python_net_import
from mastapy._math.vector_2d import Vector2D
from mastapy._math.vector_3d import Vector3D
from mastapy._math.vector_4d import Vector4D
from mastapy._math.color import Color
from mastapy._math.matrix_4x4 import Matrix4x4
from mastapy._internal.tuple_with_name import TupleWithName

_SYSTEM = python_net_import('System')
_IO = python_net_import('System.IO')
_IMAGE = python_net_import('System.Drawing.Image')
_COLOR = python_net_import('System.Drawing', 'Color')
_LIST = python_net_import('System.Collections.Generic', 'List')
_DICTIONARY = python_net_import('System.Collections.Generic', 'Dictionary')
_VECTOR2D = python_net_import('SMT.MastaAPI.MathUtility', 'Vector2D')
_VECTOR3D = python_net_import('SMT.MastaAPI.MathUtility', 'Vector3D')


class ConversionException(Exception):
    pass


# -----------------------------------------------------------------------
# - Object conversions
# -----------------------------------------------------------------------


def pn_to_mp_objects_in_iterable(iterable_of_objects, wrapper=None):
    '''Wraps up all Pythonnet objects returned in an iterable in mastapy types

    Args:
        iterable_of_objects: Iterable of Pythonnet objects
        wrapper: the wrapping mastapy type

    Returns:
        Iterable[wrapper]
    '''
    if not wrapper:
        return iterable_of_objects

    if wrapper is Vector4D:
        wrapper = Vector4D.wrap
    elif wrapper is Vector3D:
        wrapper = Vector3D.wrap
    elif wrapper is Vector2D:
        wrapper = Vector2D.wrap

    return (wrapper(x) for x in iterable_of_objects)


def pn_to_mp_objects_in_list(list_of_objects, wrapper=None):
    '''Wraps up all Pythonnet objects returned in a list in mastapy types

    Args:
        list_of_objects: List of Pythonnet objects
        wrapper: the wrapping mastapy type

    Returns:
        List[wrapper]
    '''
    if wrapper:
        return list(pn_to_mp_objects_in_iterable(list_of_objects, wrapper))
    else:
        return list(pn_to_mp_objects_in_iterable(list_of_objects))


def pn_to_mp_objects_in_iterable_of_iterables(
        iterable_of_objects, wrapper=None):
    '''Wraps up all Pythonnet objects returned in an iterable of iterables.

    Args:
        iterable_of_objects: Iterable of iterables of Pythonnet objects
        wrapper: the wrapping mastapy type

    Returns:
        Iterable[Iterable[wrapper]]
    '''
    for iterable in iterable_of_objects:
        if wrapper:
            yield from pn_to_mp_objects_in_iterable(iterable, wrapper)
        else:
            yield from pn_to_mp_objects_in_iterable(iterable)


def pn_to_mp_objects_in_list_of_lists(list_of_objects, wrapper=None):
    '''Wraps up all Pythonnet objects returned in a list of lists.

    Args:
        list_of_objects: List of lists of Pythonnet objects
        wrapper: the wrapping mastapy type

    Returns:
        List[List[wrapper]]
    '''
    if wrapper:
        return [pn_to_mp_objects_in_list(x, wrapper) for x in list_of_objects]
    else:
        return [pn_to_mp_objects_in_list(x) for x in list_of_objects]


def mp_to_pn_objects_in_list(list_of_objects):
    '''Unwraps all mastapy objects to a list of pythonnet objects

    Args:
        list_of_objects: List of mastapy objects

    Returns:
        List[x._TYPE]
    '''

    return [x.wrapped for x in list_of_objects]


def pn_to_mp_complex(pn_complex):
    '''Converts Masta API complex types to Python complex types

    Args:
        pn_complex: Masta API complex object

    Returns:
        complex
    '''

    return complex(pn_complex.Real, pn_complex.Imaginary)


def pn_to_mp_complex_list(pn_complex_list):
    '''Converts Masta API complex types in a list to Python complex types
    in a list

    Args:
        pn_complex: List of Masta API complex objects

    Returns:
        List[complex]
    '''

    return [pn_to_mp_complex(x) for x in pn_complex_list]


def pn_to_mp_enum(pn_enum):
    '''Converts C# enum to int

    Args:
        pn_enum: C# enum

    Returns:
        int
    '''
    return int(pn_enum)


def mp_to_pn_enum(mp_enum):
    '''Converts Python enum to C# enum

    Args:
        mp_enum: Python enum

    Returns:
        int
    '''

    try:
        return mp_enum.value if mp_enum else 0
    except AttributeError:
        return int(mp_enum)


def pn_to_mp_vector2d(pn_vector2d) -> Vector2D:
    '''Converts C# Vector2D and friends to a mastapy Vector2D

    Args:
        pn_vector2d: C# Vector2D (or similar)

    Returns:
        Vector2D
    '''

    return Vector2D.wrap(pn_vector2d)


def mp_to_pn_vector2d(mp_vector2d):
    '''Converts Python tuple to C# Vector2D

    Args:
        mp_vector2d: tuple

    Returns:
        C# Vector2D
    '''

    if (not hasattr(mp_vector2d, '__iter__')
            or len(mp_vector2d) != 2):
        raise ConversionException((
            'Failed to convert object to a Vector2D. '
            'Make sure that the object is iterable and contains exactly 2 '
            'components.'))

    return _VECTOR2D(mp_vector2d[0], mp_vector2d[1])


def pn_to_mp_vector3d(pn_vector3d) -> Vector3D:
    '''Converts C# Vector3D and friends to a mastapy Vector3D

    Args:
        pn_vector3d: C# Vector3D (or similar)

    Returns:
        Vector3D
    '''

    return Vector3D.wrap(pn_vector3d)


def mp_to_pn_vector3d(mp_vector3d):
    '''Converts Python tuple to C# Vector3D

    Args:
        mp_vector3d: tuple

    Returns:
        C# Vector3D
    '''

    if (not hasattr(mp_vector3d, '__iter__')
            or len(mp_vector3d) != 3):
        raise ConversionException((
            'Failed to convert object to a Vector3D. '
            'Make sure that the object is iterable and contains exactly 3 '
            'components.'))

    return _VECTOR3D(mp_vector3d[0], mp_vector3d[1], mp_vector3d[2])


def pn_to_mp_color(pn_color) -> Color:
    '''Converts C# Color to a mastapy Color

    Args:
        pn_color: C# Color

    Returns:
        Color
    '''

    return Color.wrap(pn_color)


def mp_to_pn_color(mp_color):
    '''Converts Python tuple to C# Color

    Args:
        mp_color: tuple

    Returns:
        C# Color
    '''

    num_components = len(mp_color)

    if (not hasattr(mp_color, '__iter__')
            or num_components < 3 or num_components > 4):
        raise ConversionException((
            'Failed to convert object to a Color. '
            'Make sure that the object is iterable and contains exactly 3 or 4'
            ' components.'))

    if num_components < 4:
        mp_color = *mp_color, 255

    r, g, b, a = mp_color

    return _COLOR.FromArgb(a, r, g, b)


def pn_to_mp_matrix4x4(pn_matrix4x4) -> Matrix4x4:
    '''Converts C# TransformMatrix3D to a Matrix4x4

    Args:
        pn_matrix4x4: C# TransformMatrix3D

    Returns:
        Matrix4x4
    '''

    return Matrix4x4.wrap(pn_matrix4x4)


def mp_to_pn_matrix4x4(mp_matrix4x4) -> Matrix4x4:
    '''Converts Matrix4x4 to a C# TransformMatrix3D

    Args:
        mp_matrix4x4: Matrix4x4

    Returns:
        TransformMatrix3D
    '''

    message = ('Can only pass in Matrix4x4 that was first obtained from '
               'Masta. You cannot pass in a Matrix4x4 you have constructed '
               'yourself.')

    try:
        if not mp_matrix4x4.wrapped:
            raise ConversionException(message)

        return mp_matrix4x4.wrapped
    except AttributeError:
        raise ConversionException(message)


def pn_to_mp_tuple_with_name(pn_tuple_with_name, conversion_methods=None):
    '''Converts C# NamedTuple to Python TupleWithName

    Args:
        pn_tuple_with_name: C# NamedTuple
        conversion_methods (optional): conversion methods for items in tuple

    Returns:
        TupleWithName
    '''

    attrs = filter(None, map(
        lambda x: getattr(
            pn_tuple_with_name, 'Item' + str(x), None), range(1, 8)))
    converted = (
        map(
            lambda x: x[1](x[0]) if x[1] else x[0],
            zip(attrs, conversion_methods))
        if conversion_methods
        else attrs)
    return TupleWithName(*tuple(converted), name=pn_tuple_with_name.Name)


def pn_to_mp_datetime(pn_datetime):
    '''Converts C# System.DateTime struct to python datetime object

    Args:
        pn_datetime: C# System.DateTime struct

    Returns:
        datetime
    '''

    if not pn_datetime:
        return datetime.max

    year = pn_datetime.Year
    month = pn_datetime.Month
    day = pn_datetime.Day
    hour = pn_datetime.Hour
    minute = pn_datetime.Minute
    second = pn_datetime.Second
    microsecond = pn_datetime.Millisecond * 1000

    return datetime(year, month, day, hour, minute, second, microsecond)


def pn_to_mp_image(pn_image):
    '''Converts C# System.Drawing.Image to a PIL image

    Args:
        pn_image: C# System.Drawing.Image

    Returns:
        PIL.Image
    '''

    if not pn_image:
        return None

    image_format = python_net_import('System.Drawing.Imaging.ImageFormat')

    memory_stream = _IO.MemoryStream()
    pn_image.Save(memory_stream, image_format.Png)

    byte_data = bytes(memory_stream.ToArray())
    byte_stream = BytesIO(byte_data)
    image = Image.open(byte_stream)

    memory_stream.Dispose()

    return image


def mp_to_pn_image(mp_image):
    '''Converts PIL image to a C# System.Drawing.Image

    Args:
        mp_image: PIL image

    Returns:
        C# System.Drawing.Image
    '''

    if not mp_image:
        return None

    byte_stream = BytesIO()
    mp_image.save(byte_stream, format=mp_image.format)
    byte_data = byte_stream.getvalue()

    memory_stream = _IO.MemoryStream(byte_data)
    return _IMAGE.FromStream(memory_stream)


def pn_to_mp_smt_bitmap(pn_image):
    '''Converts C# SMTBitmap to a PIL image

    Args:
        pn_image: C# SMTBitmap

    Returns:
        PIL.Image
    '''
    if not pn_image:
        return None

    return pn_to_mp_image(pn_image.ToImage())


def mp_to_pn_smt_bitmap(mp_image):
    '''Converts PIL image to a C# SMTBitmap

    Args:
        mp_image: PIL image

    Returns:
        C# SMTBitmap
    '''
    if not mp_image:
        return None

    return mp_to_pn_image(mp_image.ToImage())


def pn_to_mp_version(pn_version):
    '''Converts C# Version to Python Version

    Args:
        pn_version: C# Version

    Returns:
        packaging.version.Version
    '''

    if not pn_version:
        return None

    return Version(pn_version.ToString())


def mp_to_pn_version(mp_version):
    '''Converts Python Version to C# Version

    Args:
        mp_version: Python Version

    Returns:
        System.Version
    '''

    if not mp_version:
        return None

    return _SYSTEM.Version(str(mp_version))


# -----------------------------------------------------------------------
# - Dictionary conversions
# -----------------------------------------------------------------------


def pn_to_mp_dict(pn_dict):
    '''Converts a C# dictionary to a python dictionary

    Note:
        We assume that the key is a basic Python type.

    Args:
        pn_dict : C# dictionary

    Returns:
        Dict[TKey, TValue]
    '''

    if not pn_dict:
        return dict()

    return {kvp.Key: kvp.Value for kvp in pn_dict}


def mp_to_pn_dict_float(mp_dict):
    '''Converts a python dictionary to a C# dictionary

    Args:
        mp_dict : python dictionary

    Returns:
        C# dictionary
    '''

    if not isinstance(mp_dict, dict):
        raise ConversionException(
            'Invalid argument provided. Argument must be a dictionary.')

    new_dict = _DICTIONARY[_SYSTEM.String, _SYSTEM.Double]()
    for key, value in mp_dict.items():

        if not isinstance(key, str):
            raise ConversionException(
                'Invalid argument provided. Dictionary keys must be str.')

        if not isinstance(value, float):
            raise ConversionException(
                'Invalid argument provided. Dictionary values must be float.')

        new_dict.Add(key, value)
    return new_dict


def pn_to_mp_objects_in_list_in_ordered_dict(dict_of_objects, wrapper=None):
    '''Wraps up all Pythonnet objects returned in a list in a dictionary in
    mastapy types and returns Python list and OrderedDict structures

    Note:
        We assume that the key is a basic Python type.

    Args:
        dict_of_objects: Dictionary of lists of objects.
            e.g. {float : [PYTHON_NET_OBJECT, ...], ...}
        wrapper: the wrapping mastapy type

    Returns:
        OrderedDict[TKey, List[wrapper]]
    '''
    if wrapper:
        return OrderedDict(
            (kv.Key, [wrapper(obj) for obj in kv.Value])
            for kv in dict_of_objects)
    else:
        return OrderedDict((kv.Key, kv.Value) for kv in dict_of_objects)


def pn_to_mp_objects_in_list_in_dict(dict_of_objects, wrapper=None):
    '''Wraps up all Pythonnet objects returned in a list in a dictionary in
    mastapy types and returns Python list and dictionary structures

    Note:
        We assume that the key is a basic Python type.

    Args:
        dict_of_objects: Dictionary of lists of objects.
            e.g. {float: [PYTHON_NET_OBJECT, ...], ...}
        wrapper: the wrapping mastapy type

    Returns:
        Dict[TKey, List[wrapper]]
    '''

    if wrapper:
        return {
            kv.Key: [wrapper(obj) for obj in kv.Value]
            for kv in dict_of_objects}
    else:
        return {kv.Key: kv.Value for kv in dict_of_objects}


# -----------------------------------------------------------------------
# - List conversions
# -----------------------------------------------------------------------


def _pn_to_mp_tuple_append(values: List[Any], other, attr: str) -> bool:
    try:
        values.append(getattr(other, attr))
    except AttributeError:
        return False

    return True


def pn_to_mp_tuple(pn_tuple):
    '''Converts a C# tuple to a Python tuple

    Args:
        pn_tuple (System.Tuple[...]): C# tuple

    Returns:
        Tuple[...]
    '''

    values = []
    i = 1

    while _pn_to_mp_tuple_append(values, pn_tuple, 'Item{}'.format(i)):
        i += 1

    return tuple(values)


def mp_to_pn_tuple(mp_tuple):
    '''Converts a Python tuple to a C# tuple

    Args:
        mp_tuple (System.Tuple[...]): Python tuple

    Returns:
        System.Tuple[...]
    '''

    if mp_tuple is None or not any(mp_tuple):
        raise ConversionException(
            'Invalid argument provided. Was expecting a tuple')

    return _SYSTEM.Tuple.Create(*mp_tuple)


def pn_to_mp_bytes(pn_list: _SYSTEM.Array[_SYSTEM.Byte]) -> bytes:
    '''Converts a C# byte array to a bytes object

    Args:
        pn_list (System.Array[System.Byte]): 1D Array of bytes

    Returns:
        bytes
    '''

    return bytes([int(x) for x in pn_list])


def pn_to_mp_list_float(pn_list: _SYSTEM.Array[_SYSTEM.Double]) -> List[float]:
    '''Converts a 1D Array of Doubles to a list of floats

    Args:
        pn_list (System.Array[System.Double]): 1D Array

    Returns:
        List[float]
    '''

    return list(pn_list)


def mp_to_pn_array_float(
        mp_list: Union[
            ndarray, Sequence[float]]) -> _SYSTEM.Array[_SYSTEM.Double]:
    '''Converts a list of floats to a 1D array

    Args:
        mp_list (List[float]): List of floats

    Returns:
        System.Array[System.Double]: i.e. System.Double[]
    '''

    if mp_list is None or not any(mp_list):
        return _SYSTEM.Array.CreateInstance(_SYSTEM.Double, 0)

    if isinstance(mp_list, ndarray):
        if len(mp_list.shape) > 1:
            raise ConversionException((
                'Invalid argument provided. Argument must be a 1D array of '
                'float values.'))

    pn_list = _SYSTEM.Array.CreateInstance(_SYSTEM.Double, len(mp_list))
    for i, x in enumerate(mp_list):
        pn_list.SetValue(_SYSTEM.Double(x), i)
    return pn_list


def mp_to_pn_list_float(
        mp_list: Union[ndarray, Sequence[float]]) -> _LIST[_SYSTEM.Double]:
    '''Converts a list of floats to a 1D list

    Args:
        mp_list (List[float]): List of floats

    Returns:
        System.Collections.Generic.List[System.Double]
    '''

    if isinstance(mp_list, ndarray):
        if len(mp_list.shape) > 1:
            raise ConversionException((
                'Invalid argument provided. Argument must be a 1D array of '
                'float values.'))

    new_list = _LIST[_SYSTEM.Double]()
    for x in mp_list:
        new_list.Add(x)
    return new_list


def pn_to_mp_list_float_2d(
        pn_list: _SYSTEM.Array[_SYSTEM.Double]) -> List[List[float]]:
    '''Converts a 2D Array of Doubles to a list of lists of floats

    Args:
        pn_list (System.Array[System.Double]): 2D Array

    Returns:
        List[List[float]]
    '''

    length0 = pn_list.GetLength(0)
    length1 = pn_list.GetLength(1)

    return [
        [pn_list.GetValue(y, x) for x in range(length1)]
        for y in range(length0)]


def mp_to_pn_list_float_2d(
        mp_list: List[List[float]]) -> _SYSTEM.Array[_SYSTEM.Double]:
    '''Converts a list of lists of floats to a 2D array

    Args:
        mp_list (List[List[float]]): List of lists of floats

    Returns:
        System.Array[System.Double]: i.e. System.Double[,]
    '''

    if mp_list is None or not any(mp_list):
        return _SYSTEM.Array.CreateInstance(_SYSTEM.Double, 0, 0)

    length0 = len(mp_list)

    if not length0 or not hasattr(mp_list[0], '__iter__'):
        raise ConversionException(
            'Invalid argument provided. Argument is not a list of lists.')

    length1 = len(mp_list[0])

    pn_list = _SYSTEM.Array.CreateInstance(_SYSTEM.Double, length0, length1)
    for i, y in enumerate(mp_list):
        if not hasattr(y, '__iter__') or len(y) != length1:
            raise ConversionException((
                'Invalid argument provided. Argument must be a '
                '2d array (i.e. [[1.0, 0.0], [0.0, 1.0]])'))
        for j, x in enumerate(y):
            pn_list.SetValue(float(x), i, j)
    return pn_list



'''_1214.py

SplineJointRating
'''


from typing import List

from mastapy._internal import constructor, conversion
from mastapy.detailed_rigid_connectors.splines.ratings import _1213
from mastapy.detailed_rigid_connectors.rating import _1218
from mastapy._internal.python_net import python_net_import

_SPLINE_JOINT_RATING = python_net_import('SMT.MastaAPI.DetailedRigidConnectors.Splines.Ratings', 'SplineJointRating')


__docformat__ = 'restructuredtext en'
__all__ = ('SplineJointRating',)


class SplineJointRating(_1218.ShaftHubConnectionRating):
    '''SplineJointRating

    This is a mastapy class.
    '''

    TYPE = _SPLINE_JOINT_RATING

    __hash__ = None

    def __init__(self, instance_to_wrap: 'SplineJointRating.TYPE'):
        super().__init__(instance_to_wrap)
        self._freeze()

    @property
    def dudley_maximum_effective_length(self) -> 'float':
        '''float: 'DudleyMaximumEffectiveLength' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return self.wrapped.DudleyMaximumEffectiveLength

    @property
    def radial_load(self) -> 'float':
        '''float: 'RadialLoad' is the original name of this property.'''

        return self.wrapped.RadialLoad

    @radial_load.setter
    def radial_load(self, value: 'float'):
        self.wrapped.RadialLoad = float(value) if value else 0.0

    @property
    def moment(self) -> 'float':
        '''float: 'Moment' is the original name of this property.'''

        return self.wrapped.Moment

    @moment.setter
    def moment(self, value: 'float'):
        self.wrapped.Moment = float(value) if value else 0.0

    @property
    def torque(self) -> 'float':
        '''float: 'Torque' is the original name of this property.'''

        return self.wrapped.Torque

    @torque.setter
    def torque(self, value: 'float'):
        self.wrapped.Torque = float(value) if value else 0.0

    @property
    def angular_velocity(self) -> 'float':
        '''float: 'AngularVelocity' is the original name of this property.'''

        return self.wrapped.AngularVelocity

    @angular_velocity.setter
    def angular_velocity(self, value: 'float'):
        self.wrapped.AngularVelocity = float(value) if value else 0.0

    @property
    def axial_force(self) -> 'float':
        '''float: 'AxialForce' is the original name of this property.'''

        return self.wrapped.AxialForce

    @axial_force.setter
    def axial_force(self, value: 'float'):
        self.wrapped.AxialForce = float(value) if value else 0.0

    @property
    def number_of_cycles(self) -> 'float':
        '''float: 'NumberOfCycles' is the original name of this property.'''

        return self.wrapped.NumberOfCycles

    @number_of_cycles.setter
    def number_of_cycles(self, value: 'float'):
        self.wrapped.NumberOfCycles = float(value) if value else 0.0

    @property
    def load(self) -> 'float':
        '''float: 'Load' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return self.wrapped.Load

    @property
    def allowable_bending_stress(self) -> 'float':
        '''float: 'AllowableBendingStress' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return self.wrapped.AllowableBendingStress

    @property
    def allowable_contact_stress(self) -> 'float':
        '''float: 'AllowableContactStress' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return self.wrapped.AllowableContactStress

    @property
    def allowable_compressive_stress(self) -> 'float':
        '''float: 'AllowableCompressiveStress' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return self.wrapped.AllowableCompressiveStress

    @property
    def allowable_shear_stress(self) -> 'float':
        '''float: 'AllowableShearStress' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return self.wrapped.AllowableShearStress

    @property
    def allowable_bursting_stress(self) -> 'float':
        '''float: 'AllowableBurstingStress' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return self.wrapped.AllowableBurstingStress

    @property
    def spline_half_ratings(self) -> 'List[_1213.SplineHalfRating]':
        '''List[SplineHalfRating]: 'SplineHalfRatings' is the original name of this property.

        Note:
            This property is readonly.
        '''

        value = conversion.pn_to_mp_objects_in_list(self.wrapped.SplineHalfRatings, constructor.new(_1213.SplineHalfRating))
        return value



'''_781.py

GearContactStiffness
'''


from mastapy.gears.ltca import _789
from mastapy._internal.python_net import python_net_import

_GEAR_CONTACT_STIFFNESS = python_net_import('SMT.MastaAPI.Gears.LTCA', 'GearContactStiffness')


__docformat__ = 'restructuredtext en'
__all__ = ('GearContactStiffness',)


class GearContactStiffness(_789.GearStiffness):
    '''GearContactStiffness

    This is a mastapy class.
    '''

    TYPE = _GEAR_CONTACT_STIFFNESS

    __hash__ = None

    def __init__(self, instance_to_wrap: 'GearContactStiffness.TYPE'):
        super().__init__(instance_to_wrap)
        self._freeze()



'''_6297.py

KlingelnbergCycloPalloidHypoidGearMeshCriticalSpeedAnalysis
'''


from mastapy.system_model.connections_and_sockets.gears import _2028
from mastapy._internal import constructor
from mastapy.system_model.analyses_and_results.static_loads import _6600
from mastapy.system_model.analyses_and_results.critical_speed_analyses import _6294
from mastapy._internal.python_net import python_net_import

_KLINGELNBERG_CYCLO_PALLOID_HYPOID_GEAR_MESH_CRITICAL_SPEED_ANALYSIS = python_net_import('SMT.MastaAPI.SystemModel.AnalysesAndResults.CriticalSpeedAnalyses', 'KlingelnbergCycloPalloidHypoidGearMeshCriticalSpeedAnalysis')


__docformat__ = 'restructuredtext en'
__all__ = ('KlingelnbergCycloPalloidHypoidGearMeshCriticalSpeedAnalysis',)


class KlingelnbergCycloPalloidHypoidGearMeshCriticalSpeedAnalysis(_6294.KlingelnbergCycloPalloidConicalGearMeshCriticalSpeedAnalysis):
    '''KlingelnbergCycloPalloidHypoidGearMeshCriticalSpeedAnalysis

    This is a mastapy class.
    '''

    TYPE = _KLINGELNBERG_CYCLO_PALLOID_HYPOID_GEAR_MESH_CRITICAL_SPEED_ANALYSIS

    __hash__ = None

    def __init__(self, instance_to_wrap: 'KlingelnbergCycloPalloidHypoidGearMeshCriticalSpeedAnalysis.TYPE'):
        super().__init__(instance_to_wrap)
        self._freeze()

    @property
    def connection_design(self) -> '_2028.KlingelnbergCycloPalloidHypoidGearMesh':
        '''KlingelnbergCycloPalloidHypoidGearMesh: 'ConnectionDesign' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return constructor.new(_2028.KlingelnbergCycloPalloidHypoidGearMesh)(self.wrapped.ConnectionDesign) if self.wrapped.ConnectionDesign else None

    @property
    def connection_load_case(self) -> '_6600.KlingelnbergCycloPalloidHypoidGearMeshLoadCase':
        '''KlingelnbergCycloPalloidHypoidGearMeshLoadCase: 'ConnectionLoadCase' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return constructor.new(_6600.KlingelnbergCycloPalloidHypoidGearMeshLoadCase)(self.wrapped.ConnectionLoadCase) if self.wrapped.ConnectionLoadCase else None



'''_2361.py

IHaveFEPartHarmonicAnalysisResults
'''


from mastapy._internal.python_net import python_net_import

_I_HAVE_FE_PART_HARMONIC_ANALYSIS_RESULTS = python_net_import('SMT.MastaAPI.SystemModel.AnalysesAndResults', 'IHaveFEPartHarmonicAnalysisResults')


__docformat__ = 'restructuredtext en'
__all__ = ('IHaveFEPartHarmonicAnalysisResults',)


class IHaveFEPartHarmonicAnalysisResults:
    '''This class is a public interface.
    The class body has intentionally been left empty.
    '''



'''_130.py

FrictionNodalComponent
'''


from mastapy.nodal_analysis.nodal_entities import _136
from mastapy._internal.python_net import python_net_import

_FRICTION_NODAL_COMPONENT = python_net_import('SMT.MastaAPI.NodalAnalysis.NodalEntities', 'FrictionNodalComponent')


__docformat__ = 'restructuredtext en'
__all__ = ('FrictionNodalComponent',)


class FrictionNodalComponent(_136.NodalComponent):
    '''FrictionNodalComponent

    This is a mastapy class.
    '''

    TYPE = _FRICTION_NODAL_COMPONENT

    __hash__ = None

    def __init__(self, instance_to_wrap: 'FrictionNodalComponent.TYPE'):
        super().__init__(instance_to_wrap)
        self._freeze()



'''_422.py

CylindricalGearRating
'''


from mastapy._internal import constructor
from mastapy.gears.rating import _324, _326
from mastapy.gears.rating.cylindrical import _418, _419, _448
from mastapy._internal.cast_exception import CastException
from mastapy.gears.gear_designs.cylindrical import _950, _977
from mastapy._internal.python_net import python_net_import

_CYLINDRICAL_GEAR_RATING = python_net_import('SMT.MastaAPI.Gears.Rating.Cylindrical', 'CylindricalGearRating')


__docformat__ = 'restructuredtext en'
__all__ = ('CylindricalGearRating',)


class CylindricalGearRating(_326.GearRating):
    '''CylindricalGearRating

    This is a mastapy class.
    '''

    TYPE = _CYLINDRICAL_GEAR_RATING

    __hash__ = None

    def __init__(self, instance_to_wrap: 'CylindricalGearRating.TYPE'):
        super().__init__(instance_to_wrap)
        self._freeze()

    @property
    def damage_bending(self) -> 'float':
        '''float: 'DamageBending' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return self.wrapped.DamageBending

    @property
    def damage_contact(self) -> 'float':
        '''float: 'DamageContact' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return self.wrapped.DamageContact

    @property
    def worst_fatigue_fracture_safety_factor_with_influence_of_rim(self) -> 'float':
        '''float: 'WorstFatigueFractureSafetyFactorWithInfluenceOfRim' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return self.wrapped.WorstFatigueFractureSafetyFactorWithInfluenceOfRim

    @property
    def worst_crack_initiation_safety_factor_with_influence_of_rim(self) -> 'float':
        '''float: 'WorstCrackInitiationSafetyFactorWithInfluenceOfRim' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return self.wrapped.WorstCrackInitiationSafetyFactorWithInfluenceOfRim

    @property
    def worst_permanent_deformation_safety_factor_with_influence_of_rim(self) -> 'float':
        '''float: 'WorstPermanentDeformationSafetyFactorWithInfluenceOfRim' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return self.wrapped.WorstPermanentDeformationSafetyFactorWithInfluenceOfRim

    @property
    def left_flank_rating(self) -> '_324.GearFlankRating':
        '''GearFlankRating: 'LeftFlankRating' is the original name of this property.

        Note:
            This property is readonly.
        '''

        if _324.GearFlankRating.TYPE not in self.wrapped.LeftFlankRating.__class__.__mro__:
            raise CastException('Failed to cast left_flank_rating to GearFlankRating. Expected: {}.'.format(self.wrapped.LeftFlankRating.__class__.__qualname__))

        return constructor.new_override(self.wrapped.LeftFlankRating.__class__)(self.wrapped.LeftFlankRating) if self.wrapped.LeftFlankRating else None

    @property
    def right_flank_rating(self) -> '_324.GearFlankRating':
        '''GearFlankRating: 'RightFlankRating' is the original name of this property.

        Note:
            This property is readonly.
        '''

        if _324.GearFlankRating.TYPE not in self.wrapped.RightFlankRating.__class__.__mro__:
            raise CastException('Failed to cast right_flank_rating to GearFlankRating. Expected: {}.'.format(self.wrapped.RightFlankRating.__class__.__qualname__))

        return constructor.new_override(self.wrapped.RightFlankRating.__class__)(self.wrapped.RightFlankRating) if self.wrapped.RightFlankRating else None

    @property
    def vdi2737_safety_factor(self) -> '_448.VDI2737SafetyFactorReportingObject':
        '''VDI2737SafetyFactorReportingObject: 'VDI2737SafetyFactor' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return constructor.new(_448.VDI2737SafetyFactorReportingObject)(self.wrapped.VDI2737SafetyFactor) if self.wrapped.VDI2737SafetyFactor else None

    @property
    def cylindrical_gear(self) -> '_950.CylindricalGearDesign':
        '''CylindricalGearDesign: 'CylindricalGear' is the original name of this property.

        Note:
            This property is readonly.
        '''

        if _950.CylindricalGearDesign.TYPE not in self.wrapped.CylindricalGear.__class__.__mro__:
            raise CastException('Failed to cast cylindrical_gear to CylindricalGearDesign. Expected: {}.'.format(self.wrapped.CylindricalGear.__class__.__qualname__))

        return constructor.new_override(self.wrapped.CylindricalGear.__class__)(self.wrapped.CylindricalGear) if self.wrapped.CylindricalGear else None



'''_965.py

CylindricalGearSetDesign
'''


from typing import List, Optional

from mastapy.gears.gear_designs.cylindrical import (
    _936, _983, _1006, _1000,
    _1024, _966, _997, _950,
    _957, _967
)
from mastapy._internal import enum_with_selected_value_runtime, constructor, conversion
from mastapy._internal.implicit import list_with_selected_item, overridable
from mastapy._internal.overridable_constructor import _unpack_overridable
from mastapy.gears import _284
from mastapy.materials.efficiency import _262
from mastapy.gears.rating.cylindrical.iso6336 import _469
from mastapy._internal.python_net import python_net_import
from mastapy.gears.gear_designs.cylindrical.micro_geometry import _1039
from mastapy.gears.manufacturing.cylindrical import _577
from mastapy.gears.rating.cylindrical import _426
from mastapy.gears.gear_designs import _888

_DATABASE_WITH_SELECTED_ITEM = python_net_import('SMT.MastaAPI.UtilityGUI.Databases', 'DatabaseWithSelectedItem')
_CYLINDRICAL_GEAR_SET_DESIGN = python_net_import('SMT.MastaAPI.Gears.GearDesigns.Cylindrical', 'CylindricalGearSetDesign')


__docformat__ = 'restructuredtext en'
__all__ = ('CylindricalGearSetDesign',)


class CylindricalGearSetDesign(_888.GearSetDesign):
    '''CylindricalGearSetDesign

    This is a mastapy class.
    '''

    TYPE = _CYLINDRICAL_GEAR_SET_DESIGN

    __hash__ = None

    def __init__(self, instance_to_wrap: 'CylindricalGearSetDesign.TYPE'):
        super().__init__(instance_to_wrap)
        self._freeze()

    @property
    def profile_shift_distribution_rule(self) -> '_936.AddendumModificationDistributionRule':
        '''AddendumModificationDistributionRule: 'ProfileShiftDistributionRule' is the original name of this property.'''

        value = conversion.pn_to_mp_enum(self.wrapped.ProfileShiftDistributionRule)
        return constructor.new(_936.AddendumModificationDistributionRule)(value) if value else None

    @profile_shift_distribution_rule.setter
    def profile_shift_distribution_rule(self, value: '_936.AddendumModificationDistributionRule'):
        value = value if value else None
        value = conversion.mp_to_pn_enum(value)
        self.wrapped.ProfileShiftDistributionRule = value

    @property
    def gear_tooth_thickness_reduction_allowance(self) -> 'list_with_selected_item.ListWithSelectedItem_str':
        '''list_with_selected_item.ListWithSelectedItem_str: 'GearToothThicknessReductionAllowance' is the original name of this property.'''

        return constructor.new(list_with_selected_item.ListWithSelectedItem_str)(self.wrapped.GearToothThicknessReductionAllowance) if self.wrapped.GearToothThicknessReductionAllowance else None

    @gear_tooth_thickness_reduction_allowance.setter
    def gear_tooth_thickness_reduction_allowance(self, value: 'list_with_selected_item.ListWithSelectedItem_str.implicit_type()'):
        wrapper_type = list_with_selected_item.ListWithSelectedItem_str.wrapper_type()
        enclosed_type = list_with_selected_item.ListWithSelectedItem_str.implicit_type()
        value = wrapper_type[enclosed_type](enclosed_type(value) if value else None)
        self.wrapped.GearToothThicknessReductionAllowance = value

    @property
    def gear_tooth_thickness_tolerance(self) -> 'list_with_selected_item.ListWithSelectedItem_str':
        '''list_with_selected_item.ListWithSelectedItem_str: 'GearToothThicknessTolerance' is the original name of this property.'''

        return constructor.new(list_with_selected_item.ListWithSelectedItem_str)(self.wrapped.GearToothThicknessTolerance) if self.wrapped.GearToothThicknessTolerance else None

    @gear_tooth_thickness_tolerance.setter
    def gear_tooth_thickness_tolerance(self, value: 'list_with_selected_item.ListWithSelectedItem_str.implicit_type()'):
        wrapper_type = list_with_selected_item.ListWithSelectedItem_str.wrapper_type()
        enclosed_type = list_with_selected_item.ListWithSelectedItem_str.implicit_type()
        value = wrapper_type[enclosed_type](enclosed_type(value) if value else None)
        self.wrapped.GearToothThicknessTolerance = value

    @property
    def coefficient_of_friction_calculation_method(self) -> 'overridable.Overridable_CoefficientOfFrictionCalculationMethod':
        '''overridable.Overridable_CoefficientOfFrictionCalculationMethod: 'CoefficientOfFrictionCalculationMethod' is the original name of this property.'''

        value = overridable.Overridable_CoefficientOfFrictionCalculationMethod.wrapped_type()
        return enum_with_selected_value_runtime.create(self.wrapped.CoefficientOfFrictionCalculationMethod, value) if self.wrapped.CoefficientOfFrictionCalculationMethod else None

    @coefficient_of_friction_calculation_method.setter
    def coefficient_of_friction_calculation_method(self, value: 'overridable.Overridable_CoefficientOfFrictionCalculationMethod.implicit_type()'):
        wrapper_type = overridable.Overridable_CoefficientOfFrictionCalculationMethod.wrapper_type()
        enclosed_type = overridable.Overridable_CoefficientOfFrictionCalculationMethod.implicit_type()
        value, is_overridden = _unpack_overridable(value)
        value = conversion.mp_to_pn_enum(value)
        value = wrapper_type[enclosed_type](value if value else None, is_overridden)
        self.wrapped.CoefficientOfFrictionCalculationMethod = value

    @property
    def efficiency_rating_method(self) -> '_262.EfficiencyRatingMethod':
        '''EfficiencyRatingMethod: 'EfficiencyRatingMethod' is the original name of this property.'''

        value = conversion.pn_to_mp_enum(self.wrapped.EfficiencyRatingMethod)
        return constructor.new(_262.EfficiencyRatingMethod)(value) if value else None

    @efficiency_rating_method.setter
    def efficiency_rating_method(self, value: '_262.EfficiencyRatingMethod'):
        value = value if value else None
        value = conversion.mp_to_pn_enum(value)
        self.wrapped.EfficiencyRatingMethod = value

    @property
    def helical_gear_micro_geometry_option(self) -> '_469.HelicalGearMicroGeometryOption':
        '''HelicalGearMicroGeometryOption: 'HelicalGearMicroGeometryOption' is the original name of this property.'''

        value = conversion.pn_to_mp_enum(self.wrapped.HelicalGearMicroGeometryOption)
        return constructor.new(_469.HelicalGearMicroGeometryOption)(value) if value else None

    @helical_gear_micro_geometry_option.setter
    def helical_gear_micro_geometry_option(self, value: '_469.HelicalGearMicroGeometryOption'):
        value = value if value else None
        value = conversion.mp_to_pn_enum(value)
        self.wrapped.HelicalGearMicroGeometryOption = value

    @property
    def gear_fit_system(self) -> '_983.GearFitSystems':
        '''GearFitSystems: 'GearFitSystem' is the original name of this property.'''

        value = conversion.pn_to_mp_enum(self.wrapped.GearFitSystem)
        return constructor.new(_983.GearFitSystems)(value) if value else None

    @gear_fit_system.setter
    def gear_fit_system(self, value: '_983.GearFitSystems'):
        value = value if value else None
        value = conversion.mp_to_pn_enum(value)
        self.wrapped.GearFitSystem = value

    @property
    def helix_angle(self) -> 'float':
        '''float: 'HelixAngle' is the original name of this property.'''

        return self.wrapped.HelixAngle

    @helix_angle.setter
    def helix_angle(self, value: 'float'):
        self.wrapped.HelixAngle = float(value) if value else 0.0

    @property
    def normal_module_maintain_transverse_profile(self) -> 'float':
        '''float: 'NormalModuleMaintainTransverseProfile' is the original name of this property.'''

        return self.wrapped.NormalModuleMaintainTransverseProfile

    @normal_module_maintain_transverse_profile.setter
    def normal_module_maintain_transverse_profile(self, value: 'float'):
        self.wrapped.NormalModuleMaintainTransverseProfile = float(value) if value else 0.0

    @property
    def helix_angle_maintain_transverse_profile(self) -> 'float':
        '''float: 'HelixAngleMaintainTransverseProfile' is the original name of this property.'''

        return self.wrapped.HelixAngleMaintainTransverseProfile

    @helix_angle_maintain_transverse_profile.setter
    def helix_angle_maintain_transverse_profile(self, value: 'float'):
        self.wrapped.HelixAngleMaintainTransverseProfile = float(value) if value else 0.0

    @property
    def root_gear_profile_shift_coefficient_maintain_tip_and_root_diameters(self) -> 'float':
        '''float: 'RootGearProfileShiftCoefficientMaintainTipAndRootDiameters' is the original name of this property.'''

        return self.wrapped.RootGearProfileShiftCoefficientMaintainTipAndRootDiameters

    @root_gear_profile_shift_coefficient_maintain_tip_and_root_diameters.setter
    def root_gear_profile_shift_coefficient_maintain_tip_and_root_diameters(self, value: 'float'):
        self.wrapped.RootGearProfileShiftCoefficientMaintainTipAndRootDiameters = float(value) if value else 0.0

    @property
    def normal_pressure_angle_maintain_transverse_profile(self) -> 'float':
        '''float: 'NormalPressureAngleMaintainTransverseProfile' is the original name of this property.'''

        return self.wrapped.NormalPressureAngleMaintainTransverseProfile

    @normal_pressure_angle_maintain_transverse_profile.setter
    def normal_pressure_angle_maintain_transverse_profile(self, value: 'float'):
        self.wrapped.NormalPressureAngleMaintainTransverseProfile = float(value) if value else 0.0

    @property
    def helix_angle_calculating_gear_teeth_numbers(self) -> 'float':
        '''float: 'HelixAngleCalculatingGearTeethNumbers' is the original name of this property.'''

        return self.wrapped.HelixAngleCalculatingGearTeethNumbers

    @helix_angle_calculating_gear_teeth_numbers.setter
    def helix_angle_calculating_gear_teeth_numbers(self, value: 'float'):
        self.wrapped.HelixAngleCalculatingGearTeethNumbers = float(value) if value else 0.0

    @property
    def helix_angle_with_centre_distance_adjustment(self) -> 'float':
        '''float: 'HelixAngleWithCentreDistanceAdjustment' is the original name of this property.'''

        return self.wrapped.HelixAngleWithCentreDistanceAdjustment

    @helix_angle_with_centre_distance_adjustment.setter
    def helix_angle_with_centre_distance_adjustment(self, value: 'float'):
        self.wrapped.HelixAngleWithCentreDistanceAdjustment = float(value) if value else 0.0

    @property
    def normal_module(self) -> 'float':
        '''float: 'NormalModule' is the original name of this property.'''

        return self.wrapped.NormalModule

    @normal_module.setter
    def normal_module(self, value: 'float'):
        self.wrapped.NormalModule = float(value) if value else 0.0

    @property
    def normal_module_calculating_gear_teeth_numbers(self) -> 'float':
        '''float: 'NormalModuleCalculatingGearTeethNumbers' is the original name of this property.'''

        return self.wrapped.NormalModuleCalculatingGearTeethNumbers

    @normal_module_calculating_gear_teeth_numbers.setter
    def normal_module_calculating_gear_teeth_numbers(self, value: 'float'):
        self.wrapped.NormalModuleCalculatingGearTeethNumbers = float(value) if value else 0.0

    @property
    def normal_module_with_centre_distance_adjustment(self) -> 'float':
        '''float: 'NormalModuleWithCentreDistanceAdjustment' is the original name of this property.'''

        return self.wrapped.NormalModuleWithCentreDistanceAdjustment

    @normal_module_with_centre_distance_adjustment.setter
    def normal_module_with_centre_distance_adjustment(self, value: 'float'):
        self.wrapped.NormalModuleWithCentreDistanceAdjustment = float(value) if value else 0.0

    @property
    def diametral_pitch_per_inch(self) -> 'float':
        '''float: 'DiametralPitchPerInch' is the original name of this property.'''

        return self.wrapped.DiametralPitchPerInch

    @diametral_pitch_per_inch.setter
    def diametral_pitch_per_inch(self, value: 'float'):
        self.wrapped.DiametralPitchPerInch = float(value) if value else 0.0

    @property
    def transverse_pitch(self) -> 'float':
        '''float: 'TransversePitch' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return self.wrapped.TransversePitch

    @property
    def axial_pitch(self) -> 'float':
        '''float: 'AxialPitch' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return self.wrapped.AxialPitch

    @property
    def transverse_module(self) -> 'float':
        '''float: 'TransverseModule' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return self.wrapped.TransverseModule

    @property
    def is_asymmetric(self) -> 'bool':
        '''bool: 'IsAsymmetric' is the original name of this property.'''

        return self.wrapped.IsAsymmetric

    @is_asymmetric.setter
    def is_asymmetric(self, value: 'bool'):
        self.wrapped.IsAsymmetric = bool(value) if value else False

    @property
    def normal_pitch(self) -> 'float':
        '''float: 'NormalPitch' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return self.wrapped.NormalPitch

    @property
    def all_gears_number_of_teeth(self) -> 'List[int]':
        '''List[int]: 'AllGearsNumberOfTeeth' is the original name of this property.'''

        value = conversion.pn_to_mp_objects_in_list(self.wrapped.AllGearsNumberOfTeeth, int)
        return value

    @all_gears_number_of_teeth.setter
    def all_gears_number_of_teeth(self, value: 'List[int]'):
        value = value if value else None
        value = conversion.mp_to_pn_objects_in_list(value)
        self.wrapped.AllGearsNumberOfTeeth = value

    @property
    def parameter_for_calculating_tooth_temperature(self) -> 'float':
        '''float: 'ParameterForCalculatingToothTemperature' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return self.wrapped.ParameterForCalculatingToothTemperature

    @property
    def fe_model_for_tiff(self) -> 'str':
        '''str: 'FEModelForTIFF' is the original name of this property.'''

        return self.wrapped.FEModelForTIFF.SelectedItemName

    @fe_model_for_tiff.setter
    def fe_model_for_tiff(self, value: 'str'):
        self.wrapped.FEModelForTIFF.SetSelectedItem(str(value) if value else None)

    @property
    def minimum_axial_contact_ratio(self) -> 'float':
        '''float: 'MinimumAxialContactRatio' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return self.wrapped.MinimumAxialContactRatio

    @property
    def minimum_tip_thickness(self) -> 'float':
        '''float: 'MinimumTipThickness' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return self.wrapped.MinimumTipThickness

    @property
    def cylindrical_gear_set_micro_geometry(self) -> '_1039.CylindricalGearSetMicroGeometry':
        '''CylindricalGearSetMicroGeometry: 'CylindricalGearSetMicroGeometry' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return constructor.new(_1039.CylindricalGearSetMicroGeometry)(self.wrapped.CylindricalGearSetMicroGeometry) if self.wrapped.CylindricalGearSetMicroGeometry else None

    @property
    def scuffing(self) -> '_1006.Scuffing':
        '''Scuffing: 'Scuffing' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return constructor.new(_1006.Scuffing)(self.wrapped.Scuffing) if self.wrapped.Scuffing else None

    @property
    def micropitting(self) -> '_1000.Micropitting':
        '''Micropitting: 'Micropitting' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return constructor.new(_1000.Micropitting)(self.wrapped.Micropitting) if self.wrapped.Micropitting else None

    @property
    def usage(self) -> '_1024.Usage':
        '''Usage: 'Usage' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return constructor.new(_1024.Usage)(self.wrapped.Usage) if self.wrapped.Usage else None

    @property
    def left_flank(self) -> '_966.CylindricalGearSetFlankDesign':
        '''CylindricalGearSetFlankDesign: 'LeftFlank' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return constructor.new(_966.CylindricalGearSetFlankDesign)(self.wrapped.LeftFlank) if self.wrapped.LeftFlank else None

    @property
    def right_flank(self) -> '_966.CylindricalGearSetFlankDesign':
        '''CylindricalGearSetFlankDesign: 'RightFlank' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return constructor.new(_966.CylindricalGearSetFlankDesign)(self.wrapped.RightFlank) if self.wrapped.RightFlank else None

    @property
    def ltca_settings(self) -> '_997.LtcaSettings':
        '''LtcaSettings: 'LTCASettings' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return constructor.new(_997.LtcaSettings)(self.wrapped.LTCASettings) if self.wrapped.LTCASettings else None

    @property
    def cylindrical_gear_set_manufacturing_configuration(self) -> '_577.CylindricalSetManufacturingConfig':
        '''CylindricalSetManufacturingConfig: 'CylindricalGearSetManufacturingConfiguration' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return constructor.new(_577.CylindricalSetManufacturingConfig)(self.wrapped.CylindricalGearSetManufacturingConfiguration) if self.wrapped.CylindricalGearSetManufacturingConfiguration else None

    @property
    def gears(self) -> 'List[_950.CylindricalGearDesign]':
        '''List[CylindricalGearDesign]: 'Gears' is the original name of this property.

        Note:
            This property is readonly.
        '''

        value = conversion.pn_to_mp_objects_in_list(self.wrapped.Gears, constructor.new(_950.CylindricalGearDesign))
        return value

    @property
    def cylindrical_gears(self) -> 'List[_950.CylindricalGearDesign]':
        '''List[CylindricalGearDesign]: 'CylindricalGears' is the original name of this property.

        Note:
            This property is readonly.
        '''

        value = conversion.pn_to_mp_objects_in_list(self.wrapped.CylindricalGears, constructor.new(_950.CylindricalGearDesign))
        return value

    @property
    def cylindrical_meshes(self) -> 'List[_957.CylindricalGearMeshDesign]':
        '''List[CylindricalGearMeshDesign]: 'CylindricalMeshes' is the original name of this property.

        Note:
            This property is readonly.
        '''

        value = conversion.pn_to_mp_objects_in_list(self.wrapped.CylindricalMeshes, constructor.new(_957.CylindricalGearMeshDesign))
        return value

    @property
    def flanks(self) -> 'List[_966.CylindricalGearSetFlankDesign]':
        '''List[CylindricalGearSetFlankDesign]: 'Flanks' is the original name of this property.

        Note:
            This property is readonly.
        '''

        value = conversion.pn_to_mp_objects_in_list(self.wrapped.Flanks, constructor.new(_966.CylindricalGearSetFlankDesign))
        return value

    @property
    def micro_geometries(self) -> 'List[_1039.CylindricalGearSetMicroGeometry]':
        '''List[CylindricalGearSetMicroGeometry]: 'MicroGeometries' is the original name of this property.

        Note:
            This property is readonly.
        '''

        value = conversion.pn_to_mp_objects_in_list(self.wrapped.MicroGeometries, constructor.new(_1039.CylindricalGearSetMicroGeometry))
        return value

    @property
    def both_flanks(self) -> '_966.CylindricalGearSetFlankDesign':
        '''CylindricalGearSetFlankDesign: 'BothFlanks' is the original name of this property.

        Note:
            This property is readonly.
        '''

        return constructor.new(_966.CylindricalGearSetFlankDesign)(self.wrapped.BothFlanks) if self.wrapped.BothFlanks else None

    @property
    def manufacturing_configurations(self) -> 'List[_577.CylindricalSetManufacturingConfig]':
        '''List[CylindricalSetManufacturingConfig]: 'ManufacturingConfigurations' is the original name of this property.

        Note:
            This property is readonly.
        '''

        value = conversion.pn_to_mp_objects_in_list(self.wrapped.ManufacturingConfigurations, constructor.new(_577.CylindricalSetManufacturingConfig))
        return value

    def centre_distance_editor(self):
        ''' 'CentreDistanceEditor' is the original name of this method.'''

        self.wrapped.CentreDistanceEditor()

    def set_helix_angle_for_axial_contact_ratio(self):
        ''' 'SetHelixAngleForAxialContactRatio' is the original name of this method.'''

        self.wrapped.SetHelixAngleForAxialContactRatio()

    def create_optimiser(self, duty_cycle: '_426.CylindricalGearSetDutyCycleRating') -> '_967.CylindricalGearSetMacroGeometryOptimiser':
        ''' 'CreateOptimiser' is the original name of this method.

        Args:
            duty_cycle (mastapy.gears.rating.cylindrical.CylindricalGearSetDutyCycleRating)

        Returns:
            mastapy.gears.gear_designs.cylindrical.CylindricalGearSetMacroGeometryOptimiser
        '''

        method_result = self.wrapped.CreateOptimiser(duty_cycle.wrapped if duty_cycle else None)
        return constructor.new_override(method_result.__class__)(method_result) if method_result else None

    def set_active_manufacturing_configuration(self, value: '_577.CylindricalSetManufacturingConfig'):
        ''' 'SetActiveManufacturingConfiguration' is the original name of this method.

        Args:
            value (mastapy.gears.manufacturing.cylindrical.CylindricalSetManufacturingConfig)
        '''

        self.wrapped.SetActiveManufacturingConfiguration(value.wrapped if value else None)

    def add_new_manufacturing_configuration(self, new_config_name: Optional['str'] = 'None') -> '_577.CylindricalSetManufacturingConfig':
        ''' 'AddNewManufacturingConfiguration' is the original name of this method.

        Args:
            new_config_name (str, optional)

        Returns:
            mastapy.gears.manufacturing.cylindrical.CylindricalSetManufacturingConfig
        '''

        new_config_name = str(new_config_name)
        method_result = self.wrapped.AddNewManufacturingConfiguration(new_config_name if new_config_name else None)
        return constructor.new_override(method_result.__class__)(method_result) if method_result else None

    def clear_all_tooth_thickness_specifications(self):
        ''' 'ClearAllToothThicknessSpecifications' is the original name of this method.'''

        self.wrapped.ClearAllToothThicknessSpecifications()

    def delete_unused_manufacturing_configurations(self):
        ''' 'DeleteUnusedManufacturingConfigurations' is the original name of this method.'''

        self.wrapped.DeleteUnusedManufacturingConfigurations()

    def delete_manufacturing_configuration(self, config: '_577.CylindricalSetManufacturingConfig'):
        ''' 'DeleteManufacturingConfiguration' is the original name of this method.

        Args:
            config (mastapy.gears.manufacturing.cylindrical.CylindricalSetManufacturingConfig)
        '''

        self.wrapped.DeleteManufacturingConfiguration(config.wrapped if config else None)

    def try_make_valid(self):
        ''' 'TryMakeValid' is the original name of this method.'''

        self.wrapped.TryMakeValid()

    def add_new_micro_geometry(self) -> '_1039.CylindricalGearSetMicroGeometry':
        ''' 'AddNewMicroGeometry' is the original name of this method.

        Returns:
            mastapy.gears.gear_designs.cylindrical.micro_geometry.CylindricalGearSetMicroGeometry
        '''

        method_result = self.wrapped.AddNewMicroGeometry()
        return constructor.new_override(method_result.__class__)(method_result) if method_result else None

    def delete_micro_geometry(self, micro_geometry: '_1039.CylindricalGearSetMicroGeometry'):
        ''' 'DeleteMicroGeometry' is the original name of this method.

        Args:
            micro_geometry (mastapy.gears.gear_designs.cylindrical.micro_geometry.CylindricalGearSetMicroGeometry)
        '''

        self.wrapped.DeleteMicroGeometry(micro_geometry.wrapped if micro_geometry else None)

    def set_active_micro_geometry(self, value: '_1039.CylindricalGearSetMicroGeometry'):
        ''' 'SetActiveMicroGeometry' is the original name of this method.

        Args:
            value (mastapy.gears.gear_designs.cylindrical.micro_geometry.CylindricalGearSetMicroGeometry)
        '''

        self.wrapped.SetActiveMicroGeometry(value.wrapped if value else None)

    def micro_geometry_named(self, micro_geometry_name: 'str') -> '_1039.CylindricalGearSetMicroGeometry':
        ''' 'MicroGeometryNamed' is the original name of this method.

        Args:
            micro_geometry_name (str)

        Returns:
            mastapy.gears.gear_designs.cylindrical.micro_geometry.CylindricalGearSetMicroGeometry
        '''

        micro_geometry_name = str(micro_geometry_name)
        method_result = self.wrapped.MicroGeometryNamed(micro_geometry_name if micro_geometry_name else None)
        return constructor.new_override(method_result.__class__)(method_result) if method_result else None
